import babel from '@rollup/plugin-babel'
import commonjs from '@rollup/plugin-commonjs'
import resolve from '@rollup/plugin-node-resolve'
import typescript from '@rollup/plugin-typescript'
import terser from '@rollup/plugin-terser'
import json from '@rollup/plugin-json'

export default {
  input: 'src/index.ts',
  output: {
    file: 'lib/ajax.cjs.js',
    exports: 'default',
    format: 'cjs',
  },
  plugins: [
    resolve({
      preferBuiltins: true,
    }),
    commonjs(),
    babel({
      babelrc: false,
      presets: [
        [
          '@babel/preset-env',
          {
            targets: {
              node: '10.0',
              browsers: 'last 1 version, > 1%',
            },
          },
        ],
      ],
      plugins: [['@babel/plugin-transform-runtime', { useESModules: false }]],
      babelHelpers: 'runtime',
    }),
    typescript({
      noForceEmit: true,
    }),
    json(),
    terser(),
  ],
  external: [/@babel\/runtime/, 'axios', 'qs'],
};
