// 兼容以前的老项目写法

import { normalizeConfig } from './helpers'
import {
  AjaxInstance, AjaxOption, AnyObj, AxiosMethods
} from './types'

type OldAjaxMethods = 'query' | 'create' | 'deletes' | 'putWay' | 'patchWay'

export type CustomAjaxMethods<T extends Object, GDataShape extends AnyObj = any> = <DataType = GDataShape>(path: string, config?: Partial<AjaxOption | T>) => (data?: any, expend?: string) => Promise<DataType>

type MethodMap = {
  [k in OldAjaxMethods]: AxiosMethods
}

const methodMap: MethodMap = {
  query: 'GET',
  create: 'POST',
  putWay: 'PUT',
  deletes: 'DELETE',
  patchWay: 'PATCH'
}

export const generateCustomMethod = <T extends Object>(ctx: AjaxInstance, name: OldAjaxMethods): CustomAjaxMethods<T> => <DataType = any>(path: string, config: Partial<AjaxOption | T> = {}) => (data?: any, expand?: string) => {
  const url = expand ? `${path}/${expand}` : path;
  if (expand) {
    // eslint-disable-next-line no-console
    console.warn(`业务代码中拼接url发送请求的方式即将废弃，请使用ajax.${methodMap[name].toLocaleLowerCase()}接口替换。相关url：${url}`)
  }
  const [newParams, newData] = methodMap[name] === 'GET' ? [data, undefined] : [undefined, data]
  const config1 = normalizeConfig(ctx, {
    ...config, url, params: newParams, data: newData, method: methodMap[name],
  });

  const options = {
    ...config1,
    custom: {
      ...ctx.$options,
      ...config1,
    },
  };

  return ctx.request<any, DataType>(options);
};

function addCompatibleMethods<CusOpts>(ctx: AjaxInstance) {
  ctx.query = generateCustomMethod<CusOpts>(ctx, 'query');
  ctx.create = generateCustomMethod<CusOpts>(ctx, 'create');
  ctx.deletes = generateCustomMethod<CusOpts>(ctx, 'deletes');
  ctx.putWay = generateCustomMethod<CusOpts>(ctx, 'putWay');
  ctx.patchWay = generateCustomMethod<CusOpts>(ctx, 'patchWay');
}

export default addCompatibleMethods;
